/***************************************
  $Revision: 1.9 $

  Functions to interface with RX module (create/update rx-nodes)

  Status: NOT REVUED, NOT TESTED

 Author(s):       Andrei Robachevsky

  ******************/ /******************
  Modification History:
        andrei (17/01/2000) Created.
  ******************/ /******************
  Copyright (c) 2000                              RIPE NCC
 
  All Rights Reserved
  
  Permission to use, copy, modify, and distribute this software and its
  documentation for any purpose and without fee is hereby granted,
  provided that the above copyright notice appear in all copies and that
  both that copyright notice and this permission notice appear in
  supporting documentation, and that the name of the author not be
  used in advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.
  
  THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
  AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
  DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ***************************************/
#include "ud.h"
#include "ud_int.h"
#include <rp.h>

/* Function to fill data for radix tree */
void get_rx_data(void *element_data, void *tr_ptr)
{
Attribute_t *attr = element_data;
Transaction_t *tr = (Transaction_t *)tr_ptr;
 switch(attr->type){
   case A_IN:
   case A_RT:
   case A_I6:    
   case A_DN:
    if( RP_asc2pack( tr->packptr, attr->type, attr->value) != IP_OK ) {
     dieif( attr->type != A_DN );  /* DN can be forgiven, but radix should not be updated */
    } else tr->action |= TA_UPD_RX; /* Update radix in all other cases */
   break; 
   default: break; 
 }
}
