/***************************************
  $Revision: 1.7 $

  AK (Acknowledgement) module

  Status: NOT REVIEWED, TESTED

  Author(s):       Engin Gunduz

  ******************/ /******************
  Modification History:
        engin (10/06/2000) Created.
  ******************/ /******************
  Copyright (c) 2000                              RIPE NCC

  All Rights Reserved

  Permission to use, copy, modify, and distribute this software and its
  documentation for any purpose and without fee is hereby granted,
  provided that the above copyright notice appear in all copies and that
  both that copyright notice and this permission notice appear in
  supporting documentation, and that the name of the author not be
  used in advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO EVENT SHALL
  AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
  DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 ***************************************/

#ifndef ACK_H
#define ACK_H 

#include <stdarg.h>
#include <stdio.h>
#include <strings.h>
#include <string>
#include <unistd.h>
#include <stdlib.h>
#include <iostream.h>
#include <fstream>
#include <glib.h> 
#include <time.h>
#include "UP_util.h"
#include "UP_subject.h"


void AK_add_to_ack(const char * filename, char * fmt, ...);

void AK_add_file_to_ack(const char * ackfile, const char * filetoadd);

char * AK_ack_file_name_generate( const char * tmpdir, const char * prefix);

void AK_send_ack( const char * filename, const char * to_address, const char * mailercommand);

void AK_print_ack( const char * filename );

void AK_delete_ack( const char * filename );

void AK_log_ack( const char * filename, const char * log_file );

void AK_add_to_ack_string(const char * file_name, const string msg);


#endif

